

#include "VirtualRobotReaderExampleClient.h"


#include <memory>

#include <Eigen/Geometry>

#include <IceUtil/Time.h>

#include <VirtualRobot/Robot.h>
#include <VirtualRobot/XML/RobotIO.h>
#include <VirtualRobot/VirtualRobot.h>

#include <ArmarXCore/core/PackagePath.h>
#include <ArmarXCore/core/system/ArmarXDataPath.h>
#include <ArmarXCore/core/exceptions/local/ExpressionException.h>
#include <ArmarXCore/core/time/CycleUtil.h>
#include <ArmarXCore/core/logging/Logging.h>
#include <ArmarXCore/core/time/TimeUtil.h>
#include <ArmarXCore/core/services/tasks/PeriodicTask.h>

#include <RobotAPI/libraries/armem/client/query/Builder.h>
#include <RobotAPI/libraries/armem/client/query/query_fns.h>
#include <RobotAPI/libraries/armem/core/wm/ice_conversions.h>
#include <RobotAPI/libraries/armem/server/MemoryRemoteGui.h>
#include <RobotAPI/libraries/armem/core/Time.h>


namespace armarx::robot_state
{
    VirtualRobotReaderExampleClient::VirtualRobotReaderExampleClient() :
        virtualRobotReader(this->memoryNameSystem()) {}

    armarx::PropertyDefinitionsPtr VirtualRobotReaderExampleClient::createPropertyDefinitions()
    {
        armarx::PropertyDefinitionsPtr defs =
            new ComponentPropertyDefinitions(getConfigIdentifier());

        defs->topic(debugObserver);

        defs->optional(p.robotName, "robotName");
        defs->optional(p.updateFrequency, "updateFrequency");

        virtualRobotReader.registerPropertyDefinitions(defs);

        return defs;
    }

    std::string VirtualRobotReaderExampleClient::getDefaultName() const
    {
        return "VirtualRobotReaderExampleClient";
    }

    void VirtualRobotReaderExampleClient::onInitComponent() {}

    void VirtualRobotReaderExampleClient::onConnectComponent()
    {
        virtualRobotReader.connect();

        ARMARX_IMPORTANT << "Running virtual robot synchronization example.";

        task = new PeriodicTask<VirtualRobotReaderExampleClient>(this, &VirtualRobotReaderExampleClient::run, 1000 / p.updateFrequency);
        task->start();
    }

    void VirtualRobotReaderExampleClient::onDisconnectComponent()
    {
        task->stop();
    }

    void VirtualRobotReaderExampleClient::onExitComponent() {}

    void VirtualRobotReaderExampleClient::run()
    {
        const armem::Time now = armem::Time::Now();

        // initialize if needed
        if (virtualRobot == nullptr)
        {
            TIMING_START(getRobot);

            virtualRobot = virtualRobotReader.getRobot(p.robotName, now);

            if (virtualRobot == nullptr)
            {
                ARMARX_WARNING << deactivateSpam(1) << "Could not create virtual robot.";
                return;
            }
            // only print timing once the robot is loadable & loaded
            TIMING_END_STREAM(getRobot, ARMARX_INFO);
        }

        ARMARX_INFO << deactivateSpam(10) << "Synchronizing robot";

        TIMING_START(synchronizeRobot);
        ARMARX_CHECK(virtualRobotReader.synchronizeRobot(*virtualRobot, now));
        TIMING_END_STREAM(synchronizeRobot, ARMARX_INFO);
    }

}  // namespace armarx::robot_state
