/*
 * This file is part of ArmarX.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @package    RobotAPI::gui-plugins::SkillManagerMonitorWidgetController
 * @author     Raphael Grimm ( raphael dot grimm at kit dot edu )
 * @date       2020
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */
#pragma once

#include <stack>
#include <ArmarXCore/core/system/ImportExportComponent.h>

#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXGuiPlugin.h>
#include <ArmarXGui/libraries/ArmarXGuiBase/ArmarXComponentWidgetController.h>
#include <ArmarXGui/libraries/SimpleConfigDialog/SimpleConfigDialog.h>

#include <RobotAPI/interface/skills/SkillMemoryInterface.h>

#include <RobotAPI/gui-plugins/SkillManagerPlugin/ui_SkillManagerMonitorWidget.h>


#include <RobotAPI/libraries/aron/core/type/variant/All.h>
#include <RobotAPI/libraries/aron/core/data/variant/All.h>
#include <RobotAPI/libraries/aron/core/type/visitor/variant/VariantVisitor.h>

namespace armarx
{
    // Conversion from TreeView to aron data
    class AronTreeWidgetConverterVisitor :
            public armarx::aron::type::ConstVariantVisitor
    {
    public:
        QTreeWidgetItem* parentItem;
        int index;
        aron::data::VariantPtr createdAron;

        AronTreeWidgetConverterVisitor() = delete;
        AronTreeWidgetConverterVisitor(QTreeWidgetItem* i, int x) :
            parentItem(i), index(x)
        {}

        void visitAronVariant(const aron::type::ObjectPtr&) override;
        void visitAronVariant(const aron::type::DictPtr&) override;
        void visitAronVariant(const aron::type::PairPtr&) override;
        void visitAronVariant(const aron::type::TuplePtr&) override;
        void visitAronVariant(const aron::type::ListPtr&) override;
        void visitAronVariant(const aron::type::NDArrayPtr&) override;
        void visitAronVariant(const aron::type::MatrixPtr&) override;
        void visitAronVariant(const aron::type::QuaternionPtr&) override;
        void visitAronVariant(const aron::type::PositionPtr&) override;
        void visitAronVariant(const aron::type::OrientationPtr&) override;
        void visitAronVariant(const aron::type::PosePtr&) override;
        void visitAronVariant(const aron::type::ImagePtr&) override;
        void visitAronVariant(const aron::type::PointCloudPtr&) override;
        void visitAronVariant(const aron::type::IntEnumPtr&) override;
        void visitAronVariant(const aron::type::IntPtr&) override;
        void visitAronVariant(const aron::type::LongPtr&) override;
        void visitAronVariant(const aron::type::FloatPtr&) override;
        void visitAronVariant(const aron::type::DoublePtr&) override;
        void visitAronVariant(const aron::type::BoolPtr&) override;
        void visitAronVariant(const aron::type::StringPtr&) override;
        void visitAronVariant(const aron::type::TimePtr&) override;
        void visitUnknown(Input&) override;
    };
}


